@@ -7,16 +7,21 @@
 
 [[ -z "${_UPDATE_SETUP}" ]] && _UPDATE_SETUP="1"
 [[ -z "${_UPDATE_UEFI_SHELL}" ]] && _UPDATE_UEFI_SHELL="1"
-[[ -z "${_UPDATE_UEFI_REFIND}" ]] && _UPDATE_UEFI_REFIND="1"
+[[ -z "${_UPDATE_UEFI_GUMMIBOOT}" ]] && _UPDATE_UEFI_GUMMIBOOT="1"
 
 [[ -z "${_UPDATE_SYSLINUX}" ]] && _UPDATE_SYSLINUX="1"
 [[ -z "${_UPDATE_SYSLINUX_CONFIG}" ]] && _UPDATE_SYSLINUX_CONFIG="1"
-[[ -z "${_UPDATE_GRUB_UEFI}" ]] && _UPDATE_GRUB_UEFI="1"
-[[ -z "${_UPDATE_GRUB_UEFI_CONFIG}" ]] && _UPDATE_GRUB_UEFI_CONFIG="1"
+# [[ -z "${_UPDATE_GRUB_UEFI}" ]] && _UPDATE_GRUB_UEFI="1"
+# [[ -z "${_UPDATE_GRUB_UEFI_CONFIG}" ]] && _UPDATE_GRUB_UEFI_CONFIG="1"
 
 [[ "${_UPDATE_SYSLINUX}" == "1" ]] && _UPDATE_SYSLINUX_CONFIG="1"
 [[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _UPDATE_GRUB_UEFI_CONFIG="1"
 
+[[ -z "${_UEFI_ARCH}" ]] && _UEFI_ARCH="x86_64"
+
+[[ "${_UEFI_ARCH}" == "x86_64" ]] && _SPEC_UEFI_ARCH="x64"
+[[ "${_UEFI_ARCH}" == "i386" ]] && _SPEC_UEFI_ARCH="ia32"
+
 #############################
 
 _BASENAME="$(basename "${0}")"
@@ -92,20 +97,6 @@ echo
 [[ -e "${_ARCHBOOT_ISO_WD}/splash.png" ]] && cp -f "${_ARCHBOOT_ISO_WD}/splash.png" "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/splash.png"
 echo
 
-_rename_old_files() {
-	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{vmlts,vmlinuz_i686_lts} || true
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{vmlinuz,vmlinuz_i686} || true
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{initrd.img,initramfs_i686.img} || true
-	echo
-	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{vm64lts,vmlinuz_x86_64_lts} || true
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{vm64,vmlinuz_x86_64} || true
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/boot"/{initrd64.img,initramfs_x86_64.img} || true
-	echo
-	
-}
-
 _update_syslinux_iso_files() {
 	
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux"/*.{com,bin,c32} || true
@@ -262,74 +253,108 @@ _download_uefi_shell_tianocore() {
 	
 	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/"
 	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/shell/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" || true
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/shell/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" || true
+	
 	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/EFI/shell/" || true
 	echo
 	
 	## Download Tianocore UDK/EDK2 ShellBinPkg UEFI x86_64 "Full Shell" - For Spec. Ver. >=2.3 systems
 	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi.backup" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi.backup" || true
 	echo
 	
-	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64.efi" ]]; then
-		cp -f "${_ARCHBOOT_ISO_WD}/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi"
+	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64_v2.efi" ]]; then
+		cp -f "${_ARCHBOOT_ISO_WD}/shellx64_v2.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi"
 		echo
 	else
-		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi" || true
+		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi" || true
 		echo
 		
-		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" | grep 'executable')" ]]; then
-			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" || true
-			mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi" || true
+		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" | grep 'executable')" ]]; then
+			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" || true
+			mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi" || true
 		fi
 	fi
 	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64.efi.backup" || true
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v2.efi.backup" || true
 	echo
 	
 	## Download Tianocore UDK/EDK2 EdkShellBinPkg UEFI x86_64 "Full Shell" - For Spec. Ver. <2.3 systems
 	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi.backup" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi.backup" || true
 	echo
 	
-	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64_old.efi" ]]; then
-		cp -f "${_ARCHBOOT_ISO_WD}/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi"
+	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64_v1.efi" ]]; then
+		cp -f "${_ARCHBOOT_ISO_WD}/shellx64_v1.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi"
 		echo
 	else
-		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi" || true
+		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi" || true
 		echo
 		
-		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" | grep 'executable')" ]]; then
-			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" || true
-			mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi" || true
+		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" | grep 'executable')" ]]; then
+			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" || true
+			mv "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi" || true
 		fi
 	fi
 	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_old.efi.backup" || true
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/tools/shellx64_v1.efi.backup" || true
 	echo
 	
 }
 
-_update_uefi_refind_sourceforge() {
+_update_uefi_gummiboot_USB_files() {
 	
-	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/packages/" || true
-	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/packages/refind-bin.zip" || true
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/" || true
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/loader/" || true
 	echo
 	
-	## Download latest rEFInd bin archive from sourceforge
-	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/packages/refind-bin.zip" -L "http://sourceforge.net/projects/refind/files/latest/download"
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot"
+	cp -f "/usr/lib/gummiboot/gummiboot${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/boot${_SPEC_UEFI_ARCH}.efi"
+	cp -f "/usr/lib/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/efilinux${_SPEC_UEFI_ARCH}.efi"
+	echo
+	
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/"
+	echo
+	
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/loader.conf"
+timeout 3
+default archboot-${_UEFI_ARCH}
+EOF
+	echo
+	
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/archboot-${_UEFI_ARCH}.conf"
+title   Arch Linux ${_UEFI_ARCH} archboot
+linux   /boot/vmlinuz_${_UEFI_ARCH}
+initrd  /boot/initramfs_${_UEFI_ARCH}.img
+options gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
+EOF
+	echo
+	
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/archboot-${_UEFI_ARCH}-lts.conf"
+title   Arch Linux LTS ${_UEFI_ARCH} archboot
+efi     /EFI/boot/efilinux${_SPEC_UEFI_ARCH}.efi
+options -f \\boot\\vmlinuz_x86_64_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH} initrd=\\boot\\initramfs_${_UEFI_ARCH}.img
+EOF
+	echo
+	
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/uefi-shell-${_UEFI_ARCH}-v2.conf"
+title   UEFI ${_UEFI_ARCH} Shell v2
+efi     /EFI/tools/shell${_SPEC_UEFI_ARCH}_v2.efi
+EOF
+	echo
+	
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/uefi-shell-${_UEFI_ARCH}-v1.conf"
+title   UEFI ${_UEFI_ARCH} Shell v1
+efi     /EFI/tools/shell${_SPEC_UEFI_ARCH}_v1.efi
+EOF
+	echo
 	
 }
 
-_update_grub_uefi_arch_specific_iso_files() {
-	
-	[[ "${_UEFI_ARCH}" == "x86_64" ]] && _SPEC_UEFI_ARCH="x64"
-	[[ "${_UEFI_ARCH}" == "i386" ]] && _SPEC_UEFI_ARCH="ia32"
+_update_grub_uefi_arch_specific_CD_files() {
 	
 	rm -f "${grub_uefi_mp}/EFI/boot/boot${_SPEC_UEFI_ARCH}.efi" || true
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/boot${_SPEC_UEFI_ARCH}.efi" || true
 	echo
 	
 	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/${_UEFI_ARCH}-efi" || true
@@ -378,17 +403,11 @@ EOF
 	
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub.cfg"
 	
-	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/"
-	cp -f "${grub_uefi_mp}/EFI/boot/boot${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot/boot${_SPEC_UEFI_ARCH}.efi"
-	
 	echo
 	
-	unset _UEFI_ARCH
-	unset _SPEC_UEFI_ARCH
-	
 }
 
-_update_grub_uefi_iso_files() {
+_update_grub_uefi_CD_files() {
 	
 	grub_uefi_mp="$(mktemp -d /tmp/grub_uefi_mp.XXX)"
 	
@@ -396,7 +415,6 @@ _update_grub_uefi_iso_files() {
 	echo
 	
 	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub"
-	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/EFI/boot"
 	echo
 	
 	# Create a blank image to be converted to ESP IMG
@@ -422,7 +440,7 @@ _update_grub_uefi_iso_files() {
 	echo
 	
 	_UEFI_ARCH="x86_64"
-	_update_grub_uefi_arch_specific_iso_files
+	_update_grub_uefi_arch_specific_CD_files
 	echo
 	
 	# umount images and loop
@@ -455,7 +473,7 @@ _update_grub_uefi_iso_files() {
 	
 }
 
-_update_grub_uefi_iso_config() {
+_update_grub_uefi_CD_config() {
 	
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg" || true
 	
@@ -584,14 +602,14 @@ EOF
 
 if [ "\${grub_platform}" == "efi" ]; then
 
-    menuentry "UEFI \${_UEFI_ARCH} Shell 2.0 - For Spec. Ver. >=2.3 systems" {
+    menuentry "UEFI \${_UEFI_ARCH} Shell v2 - For Spec. Ver. >=2.3 systems" {
         set root="\${archboot}"
-        chainloader /EFI/tools/shell\${_SPEC_UEFI_ARCH}.efi
+        chainloader /EFI/tools/shell\${_SPEC_UEFI_ARCH}_v2.efi
     }
 
-    menuentry "UEFI \${_UEFI_ARCH} Shell 1.0 - For Spec. Ver. <2.3 systems" {
+    menuentry "UEFI \${_UEFI_ARCH} Shell v1 - For Spec. Ver. <2.3 systems" {
         set root="\${archboot}"
-        chainloader /EFI/tools/shell\${_SPEC_UEFI_ARCH}_old.efi
+        chainloader /EFI/tools/shell\${_SPEC_UEFI_ARCH}_v1.efi
     }
 
 fi
@@ -681,8 +699,6 @@ _download_pkgs() {
 	
 }
 
-_rename_old_files
-
 [[ "${_REMOVE_i686}" == "1" ]] && _remove_i686_iso_files
 
 [[ "${_REMOVE_x86_64}" == "1" ]] && _remove_x86_64_iso_files
@@ -712,19 +728,19 @@ fi
 
 [[ "${_UPDATE_UEFI_SHELL}" == "1" ]] && _download_uefi_shell_tianocore
 
-[[ "${_UPDATE_UEFI_REFIND}" == "1" ]] && _update_uefi_refind_sourceforge
+[[ "${_UPDATE_UEFI_GUMMIBOOT}" == "1" ]] && _update_uefi_gummiboot_USB_files
 
 [[ "${_UPDATE_SYSLINUX}" == "1" ]] && _update_syslinux_iso_files
 
 [[ "${_UPDATE_SYSLINUX_CONFIG}" == "1" ]] && _update_syslinux_iso_config
 
-[[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _update_grub_uefi_iso_files
+# [[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _update_grub_uefi_CD_files
 
-[[ "${_UPDATE_GRUB_UEFI_CONFIG}" == "1" ]] && _update_grub_uefi_iso_config
+# [[ "${_UPDATE_GRUB_UEFI_CONFIG}" == "1" ]] && _update_grub_uefi_CD_config
 
 cd "${_ARCHBOOT_ISO_WD}/"
 
-## Generate the BIOS+UEFI+ISOHYBRID ISO image using xorriso (extra/libisoburn package) in mkisofs emulation mode
+## Generate the BIOS+ISOHYBRID CD image using xorriso (extra/libisoburn package) in mkisofs emulation mode
 echo "Generating the modified ISO ..."
 xorriso -as mkisofs \
 	-iso-level 3 -rock -joliet \
@@ -736,11 +752,13 @@ xorriso -as mkisofs \
 	-eltorito-boot boot/syslinux/isolinux.bin \
 	-eltorito-catalog boot/syslinux/boot.cat \
 	-no-emul-boot -boot-load-size 4 -boot-info-table \
-	-eltorito-alt-boot --efi-boot boot/grub/grub_uefi_x86_64.bin -no-emul-boot \
 	-isohybrid-mbr /usr/lib/syslinux/isohdpfx.bin \
 	-output "${_ARCHBOOT_ISO_UPDATED_PATH}" "${_ARCHBOOT_ISO_EXT_DIR}/" &> "/tmp/archboot_update_xorriso.log"
 echo
 
+## Add below line to above xorriso if UEFI CD support is required
+# -eltorito-alt-boot --efi-boot boot/grub/grub_uefi_x86_64.bin -no-emul-boot \
+
 set +x
 
 if [[ -e "${_ARCHBOOT_ISO_UPDATED_PATH}" ]]; then
@@ -757,16 +775,18 @@ else
 	echo
 fi
 
-
+unset _UEFI_ARCH
+unset _SPEC_UEFI_ARCH
 unset _REMOVE_i686
 unset _REMOVE_x86_64
 unset _UPDATE_SETUP
 unset _UPDATE_UEFI_SHELL
-unset _UPDATE_UEFI_REFIND
+unset _UPDATE_UEFI_REFIND_BIN
+unset _UPDATE_UEFI_GUMMIBOOT
 unset _UPDATE_SYSLINUX
 unset _UPDATE_SYSLINUX_CONFIG
-unset _UPDATE_GRUB_UEFI
-unset _UPDATE_GRUB_UEFI_CONFIG
+# unset _UPDATE_GRUB_UEFI
+# unset _UPDATE_GRUB_UEFI_CONFIG
 unset _ARCHBOOT_ISO_OLD_PATH
 unset _ARCHBOOT_ISO_WD
 unset _ARCHBOOT_ISO_OLD_NAME
